@@ -1,6 +1,6 @@
 # zero inflation
 hurdlemsm.fit.control <- function(
-  predmethod=c("components", "catprobs")
+  predmethod=rev(c("components", "catprobs"))
 ){
   #' @title Control of fitting parameters for zero inflated MSMs
   #' @description this is an internal function called by 
@@ -405,8 +405,8 @@ qgcomp.hurdle.noboot <- function(f,
     res$zstat <- tstat
     res$pval <- pvalz
   #}
-  attr(res, "class") <- "qgcompfit"
-  res
+    attr(res, "class") <- c("ziqgcompfit", "qgcompfit")
+    res
 }
 
 qgcomp.hurdle.boot <- function(f, 
@@ -736,7 +736,7 @@ qgcomp.hurdle.boot <- function(f,
   )
   res$zstat <- tstat
   res$pval <- pvalz
-  attr(res, "class") <- "qgcompfit"
+  attr(res, "class") <- c("ziqgcompfit", "qgcompfit")
   res
 }
 
